/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */


#define ETHERX_NAMESPACE "http://etherx.jabber.org/"

#define EPACKET_UNKNOWN -1
#define EPACKET_NORMAL 0 /* default */
#define EPACKET_REGISTRY 1 /* special flag that this is a registry packet */
#define EPACKET_ERROR 2 /* special flag that this is an error/bounced packet */
#define EPACKET_ALIAS 3 /* this is a special packet that identifies a host alias for a transport */
#define EPACKET_TRANSPORT 4 /* transport's identification phase */

#define EHANDLER_ERROR 0
#define EHANDLER_DSO 1
#define EHANDLER_SOCKET 2


typedef struct epacket_struct
{
    xmlnode x;
    char *to;
    char *from;
    char *ns;
    char *error;
    int type;
} _epacket, *epacket;

typedef void (*etherx_handler)(epacket e);

typedef struct ehandler_struct
{
    int type;
    char *host;
    char *namespace;
    etherx_handler h;
    thread t; /* used if it's an xml stream */
    pool p;

    /* these are used within etherxd */
    struct ehandler_struct *next, *children;
    int count;

} _ehandler, *ehandler;

/* public functions */
void ehandler_send(ehandler eh, xmlnode x, char *to); /* tells etherx (via ehandler) to send x to to */
ehandler ehandler_new(etherx_handler h, char *host, char *namespace); /* creates a new ehandler for this host/namespace */
void epacket_error(epacket e, char *error); /* simply turns a normal epacket into an error one */

/* socket based transport */
void etherx_init(char *secret, char *host);

/* dso stuff for etherxd */
void _etherx_dso(void *send, void *reg); /* called by etherxd to setup etherxd internal pointers/data */

/* internal functions */
ehandler _ehandler_new(char *host, char *namespace);
void _etherx_connect(ehandler eh, char *host, char *namespace);
void _etherx_receive(thread t, xmlnode x, int flag); /* callback for xmlstream when doing socket connects */
void _etherx_cleanup(void *arg); /* attempts to reconnect to etherxd */
epacket epacket_new(xmlnode x, char *to, char *from, char *ns); /* if to/from are null it extracts them from x, sets type based on element name */
xmlnode epacket_serialize(epacket e); /* puts etherx:* attribs into the xml */
void epacket_deserialize(epacket e); /* puts etherx:* attribs into the epacket and removes them from the xml */

